const std = @import("std");
const mem = std.mem;
const heap = std.heap;

const c = @import("libvirt-c.zig").c;
const err = @import("libvirt-error.zig");
const VirError = err.VirError;
const Snapshot = @This();

ptr: c.virDomainSnapshotPtr,
arena: *heap.ArenaAllocator,

pub fn init(ptr: c.virDomainSnapshotPtr, allocator: mem.Allocator) Snapshot {
    var arena = heap.ArenaAllocator.init(allocator);
    return .{
        .ptr = ptr,
        .arena = &arena,
    };
}
pub fn deinit(self: *const Snapshot) void {
    self.arena.deinit();
}

pub fn hasCurrentSnapshot() void {}
pub fn listAllSnapshots() void {}
pub fn revertToSnapshot() void {}
pub fn createXML() void {}
pub fn current() void {}

pub fn delete() void {}
pub fn free(self: *const Snapshot) VirError!void {
    if (c.virDomainSnapshotFree(self.ptr) < 0) return err.handleError();
}
pub fn getConnect() void {}
pub fn getDomain() void {}
pub fn getName() void {}
pub fn getParent() void {}
pub fn getXMLDesc() void {}
pub fn hasMetadata() void {}
pub fn isCurrent() void {}

pub fn listAllChildren() void {}
pub fn listChildrenNames() void {}
pub fn listNames() void {}

pub fn lookupByName() void {}
pub fn num() void {}
pub fn numChildren() void {}
pub fn ref(self: *const Snapshot) VirError!void {
    if (c.virDomainSnapshotRef(self.ptr) < 0) return err.handleError();
}

pub const ListFlags = enum(c_uint) {
    Descendants = c.VIR_DOMAIN_SNAPSHOT_LIST_DESCENDANTS,
    Roots = c.VIR_DOMAIN_SNAPSHOT_LIST_ROOTS,
    Metadata = c.VIR_DOMAIN_SNAPSHOT_LIST_METADATA,
    Leaves = c.VIR_DOMAIN_SNAPSHOT_LIST_LEAVES,
    NoLeaves = c.VIR_DOMAIN_SNAPSHOT_LIST_NO_LEAVES,
    NoMetadata = c.VIR_DOMAIN_SNAPSHOT_LIST_NO_METADATA,
    Inactive = c.VIR_DOMAIN_SNAPSHOT_LIST_INACTIVE,
    Active = c.VIR_DOMAIN_SNAPSHOT_LIST_ACTIVE,
    DiskOnly = c.VIR_DOMAIN_SNAPSHOT_LIST_DISK_ONLY,
    Internal = c.VIR_DOMAIN_SNAPSHOT_LIST_INTERNAL,
    External = c.VIR_DOMAIN_SNAPSHOT_LIST_EXTERNAL,
    Topological = c.VIR_DOMAIN_SNAPSHOT_LIST_TOPOLOGICAL,
};
pub const CreateFlags = enum(c_uint) {
    Redefine = c.VIR_DOMAIN_SNAPSHOT_CREATE_REDEFINE,
    Current = c.VIR_DOMAIN_SNAPSHOT_CREATE_CURRENT,
    NoMetadata = c.VIR_DOMAIN_SNAPSHOT_CREATE_NO_METADATA,
    Halt = c.VIR_DOMAIN_SNAPSHOT_CREATE_HALT,
    DiskOnly = c.VIR_DOMAIN_SNAPSHOT_CREATE_DISK_ONLY,
    ReuseExt = c.VIR_DOMAIN_SNAPSHOT_CREATE_REUSE_EXT,
    Quiesce = c.VIR_DOMAIN_SNAPSHOT_CREATE_QUIESCE,
    Atomic = c.VIR_DOMAIN_SNAPSHOT_CREATE_ATOMIC,
    Live = c.VIR_DOMAIN_SNAPSHOT_CREATE_LIVE,
    Validate = c.VIR_DOMAIN_SNAPSHOT_CREATE_VALIDATE,
};
pub const DeleteFlags = enum(c_uint) {
    Children = c.VIR_DOMAIN_SNAPSHOT_DELETE_CHILDREN,
    MetadataOnly = c.VIR_DOMAIN_SNAPSHOT_DELETE_METADATA_ONLY,
    ChildrenOnly = c.VIR_DOMAIN_SNAPSHOT_DELETE_CHILDREN_ONLY,
};
pub const RevertFlags = enum(c_uint) {
    Running = c.VIR_DOMAIN_SNAPSHOT_REVERT_RUNNING,
    Paused = c.VIR_DOMAIN_SNAPSHOT_REVERT_PAUSED,
    Force = c.VIR_DOMAIN_SNAPSHOT_REVERT_FORCE,
    ResetNvram = c.VIR_DOMAIN_SNAPSHOT_REVERT_RESET_NVRAM,
};
pub const XMLFlags = enum(c_uint) {
    Secure = c.VIR_DOMAIN_XML_SECURE,
};
